from django.db import models
from .core import VModel, VTree
from django.contrib.auth.hashers import check_password


class Role(VModel):
    duty = models.TextField(null=True, verbose_name='职责描述')

    class Meta:
        verbose_name = '角色'


class Account(VModel):
    """账户核心信息"""
    username = models.CharField(max_length=255, null=True, unique=True, verbose_name='账号')
    password = models.CharField(max_length=255, null=True, verbose_name='密码')
    email = models.CharField(max_length=255, null=True, unique=True, verbose_name='邮箱')
    """权限"""
    is_admin = models.BooleanField(default=False, verbose_name='超级管理员')
    roles = models.ManyToManyField(Role)
    """密码找回"""
    is_active = models.BooleanField(default=False, verbose_name='激活状态')
    token = models.CharField(max_length=255, null=True, verbose_name='Token')

    def is_auth(self, password):
        return password == self.token or check_password(password, self.password)

    class Meta:
        verbose_name = '账户信息'


class Menu(VTree):
    scope = models.CharField(max_length=100, null=True, verbose_name='域')
    path = models.CharField(max_length=255, null=True, verbose_name='地址')
    roles = models.ManyToManyField(Role)
    is_admin = models.BooleanField(null=True, default=False, verbose_name='超管权限')
    is_auth = models.BooleanField(null=True, default=False, verbose_name='需要登录')

    class Meta:
        verbose_name = '菜单'
        unique_together = ('scope', 'path')


class AbstractUser(VModel):
    account = models.OneToOneField(Account, null=True, on_delete=models.SET_NULL)

    class Meta:
        abstract = True



import json

from django.db.models import Count, Q

from ..classes.valar_response import ValarResponse
from ..dao import Dao
from ..dao.abstract import AbstractField

from ..dao.meta import ValarMeta
from ..models.meta import MetaView, Meta, MetaField


def upload_frame(request):
    body = json.loads(request.body)
    db = body.get('db')
    entity = body.get('entity')
    code = body.get('code')
    fields = MetaField.objects.filter(
        view__meta__db=db,
        view__meta__entity=entity,
        view__code=code,
        domain__in=[
            'ManyToManyField',
            'ManyToManyRel',
            'ManyToOneRel',
            'OneToOneField',
            'OneToOneRel',
            'ForeignKey',
            'CharField'
        ]
    ).filter(Q(allow_upload=True) | Q(allow_update=True))
    frame = {}
    for field in fields:
        prop = field.prop
        refer = field.refer
        db = refer['db']
        entity = refer['entity']
        value = refer['value']
        label = refer['label']
        try:
            includes = {prop: refer['includes'][prop] for prop in refer['includes'] if not prop.startswith('$')}
            excludes = {prop: refer['excludes'][prop] for prop in refer['excludes'] if not prop.startswith('$')}
            values = Dao(entity, db).manager.filter(**includes).exclude(**excludes).values(*[value, label])
            frame[prop] = [
                {"value": row[value], "label": row[label]}
                for row in values if row[value]
            ]
        except Exception:
            pass

    return ValarResponse(frame)


def meta_view(request, db, entity):
    body = json.loads(request.body)
    code = body.get('code')
    meta = ValarMeta(db, entity, code)
    _view = meta.meta_view()
    return ValarResponse(_view)


def load_customs(request):
    body = json.loads(request.body)
    db = body.get('db')
    entity = body.get('entity')
    code = body.get('code')
    ps = MetaField.objects.filter(
        view__code=code, view__meta__db=db, view__meta__entity=entity,
        domain='Custom').values('prop')
    current = [p.get('prop') for p in ps]
    return ValarResponse(current)


def get_fields(request):
    body = json.loads(request.body)
    db = body.get('db')
    entity = body.get('entity')
    route = body.get('route')
    dao = Dao(entity, db)
    array = []
    for prop in dao.fields:
        field: AbstractField = dao.fields.get(prop)
        if field is None:
            continue
        domain: str = field.domain
        if domain is None or domain.startswith('ManyTo'):
            continue
        elif domain in ['ForeignKey', 'OneToOneRel', 'OneToOneField']:
            refer = field.refer or {}
            _db = refer.get('db')
            _entity = refer.get('entity')
            _dao = Dao(_entity, _db)
            if f'{_db}.{_entity}' in route:
                continue
            array.append({
                'label': field.label,
                'value': field.prop.replace('_id', ''),
                'isLeaf': False,
                'db': _db,
                'entity': _entity,
            })
        elif len(route) > 1:
            array.append({
                'label': field.label,
                'value': field.prop,
                'isLeaf': True,
            })
    return ValarResponse(array)


def save_custom(request):
    return ValarResponse(True)


def add_fields(request):
    body = json.loads(request.body)
    view_id = body.get('view_id')
    props = body.get('props')
    view_dao = Dao('valar.MetaView')
    field_dao = Dao('valar.MetaField')
    view: MetaView = view_dao.find_one(view_id)
    entity = view.meta.entity
    db = view.meta.db
    dao = Dao(entity, db)
    for prop in props:
        field: AbstractField = dao.fields.get(prop)
        if field:
            _field = field.to_dict()
            _field['view_id'] = view_id
            field_dao.save_one(_field)

    return ValarResponse(True)


def metas(request):
    values = Meta.objects.all().values('db', 'entity', 'name', 'tree')
    mapping = {'orm': {'valar': []}, 'mon': []}
    for row in values:
        db = row['db']
        label = row['name']
        value = row['entity']
        tree = row['tree']
        icon = 'folder-tree' if tree else 'table'
        row.update({'label': label, 'value': value, 'icon': icon})
        if db == 'orm':
            app, _ = value.split('.')
            array = mapping['orm'].get(app, [])
            array.append(row)
            mapping['orm'][app] = array
        elif db == 'mon':
            mapping['mon'].append(row)

    mon = {'label': 'MongoDB', 'value': 'mon', 'icon': 'boxes', 'children': mapping['mon']}
    orm = {
        'label': 'SQL', 'value': 'orm', 'icon': 'boxes',
        'children': [
            {'label': app, 'value': app, 'icon': 'box', 'children': mapping['orm'][app]}
            for app in mapping['orm']
        ]
    }

    return ValarResponse([orm, mon])



import asyncio

from ..channels.sender import ValarChannelSender
import traceback


async def execute_channel(method, sender: ValarChannelSender):
    thread = asyncio.to_thread(__execute__, method, sender)
    asyncio.create_task(thread)


def __execute__(method, sender: ValarChannelSender):
    sender.start()
    try:
        response = method(sender)
        sender.done(response)
        sender.stop()
    except Exception as e:
        traceback.print_exc()
        sender.error(str(e))



import datetime
import json

from django.core.paginator import Paginator
from django.db.models.options import Options

from .abstract import AbstractDao
from .query import Query
from django.db.models import ManyToOneRel, ForeignKey, ManyToManyRel, ManyToManyField, OneToOneField, OneToOneRel, \
    CharField, BooleanField, FloatField, IntegerField, BigAutoField
from django.db.models import QuerySet
from django.db.models import Manager
from django.db.models.fields.files import FieldFile
from django.forms import FileField

from ..dao.orm_field import OrmField
from ..models.core import VTree, VModel
from ..models.meta import MetaField


class OrmDao(AbstractDao):

    def __init__(self, entity):
        model = self.engine.get_orm_model(entity)
        meta = getattr(model, '_meta')
        self.minio = self.engine.get_minio_bucket(entity)
        self.db = 'orm'
        self.entity = entity
        self.name = meta.verbose_name
        self.is_tree = issubclass(model, VTree)
        self.manager: Manager = model.objects
        fields = {}
        model_fields = meta.get_fields()
        for model_field in model_fields:
            meta_field = OrmField(entity, model_field, self.is_tree)
            fields[meta_field.prop] = meta_field
        self.fields = fields

    def save_many(self, array: list):
        results = []
        for item in array:
            item.update({'saved': True})
            results.append(self.save_one(item))
        return results

    def values(self, conditions, props: list):
        query_set, _ = self.find(conditions)
        if 'id' not in props:
            props.append('id')
        return list(query_set.values(*props))

    def insert_one(self, item):
        return self.__save__(item)

    def update_one(self, item):
        pass

    def save_one(self, item: dict):
        item.update({'saved': True})
        return self.__save__(item)

    def __save__(self, item: dict):
        oid, simple_item, complex_item = self.__detach_item__(item)
        query_set = self.manager.filter(id=oid) if oid else []
        if len(query_set):
            simple_item['modify_time'] = datetime.datetime.now()
            query_set.update(**simple_item)
            bean = query_set.first()
        else:
            if oid:
                simple_item.update({'id': oid})
            bean = self.manager.create(**simple_item)
            if simple_item.get('sort') is None:
                bean.sort = bean.id
                bean.save()
        self.__save_complex_field__(complex_item, bean)
        bean.save()
        return bean

    def delete_one(self, _id):
        oid = self.object_id(_id)
        flag = oid is not None
        if flag:
            query_set = self.manager.filter(id=oid)
            paths = self.__get_file_paths__(query_set)
            for path in paths:
                self.minio.remove(path)
            query_set.delete()
        return flag

    def find_one(self, _id):
        oid = self.object_id(_id)
        return self.manager.filter(id=oid).first() if oid is not None else None

    def find(self, conditions=None, orders=None, size=0, page=1):

        includes, excludes, orders = Query(conditions, orders).orm()
        query_set = self.manager.filter(includes).exclude(excludes).order_by(*orders)
        total = query_set.count()
        if size:
            paginator = Paginator(query_set, size)
            query_set = paginator.page(page).object_list
        return query_set, total

    def update(self, template, conditions):
        flag = template is not None and len(template.keys())
        if flag:
            oid, simple_item, complex_item = self.__detach_item__(template)
            query_set, total = self.find(conditions)
            query_set.update(**simple_item)
        return flag

    def delete(self, conditions=None) -> list:
        query_set, total = self.find(conditions)
        query_set.delete()
        return self.__get_file_paths__(query_set)

    def serialize(self, o, code=None):
        return self.__to_dict__(o, code) if isinstance(o, QuerySet) else o.full()

    def tree(self, root, conditions=None):
        all_set, _ = self.find()
        if Query(conditions).is_empty:
            return all_set
        values = all_set.values('id', 'pid')
        mapping = {item['id']: item['pid'] for item in values}
        results, _ = self.find(conditions)
        id_set = {root}
        for item in results:
            _id = item.id
            route = []
            while _id is not None:
                route.append(_id)
                _id = mapping.get(_id)
            if root in route:
                id_set.update(route)
        return all_set.filter(id__in=id_set).order_by('-sort')

    """ 以下为私有方法 """

    def __detach_item__(self, item):
        _id = item.get('id')
        if _id:
            del item['id']
        simple_item = {}
        complex_item = {}
        for prop in item:
            meta_field = self.fields.get(prop)
            if meta_field:
                value = item.get(prop)
                if meta_field.domain in ['ManyToOneRel', 'ManyToManyField', 'ManyToManyRel']:
                    complex_item[prop] = value
                elif meta_field.domain in ['OneToOneRel', 'OneToOneField', 'FileField']:
                    complex_item[prop] = value
                else:
                    simple_item[prop] = value
        return self.object_id(_id), simple_item, complex_item

    def __save_complex_field__(self, complex_item, bean):
        for prop in complex_item:
            value = complex_item[prop]
            model_field = self.fields[prop].model_field
            clazz = type(model_field)
            if clazz == ManyToManyField:
                m2m = getattr(bean, prop)
                m2m.clear()
                m2m.add(*value)
            elif clazz == ManyToOneRel:
                pass
                # getattr(bean, model_field.get_accessor_name()).clear()
                # remote_model: VModel = model_field.related_model
                # new_set: QuerySet = remote_model.objects.filter(id__in=value)
                # remote_field: ForeignKey = model_field.remote_field
                # k = remote_field.get_attname()
                # new_set.update(**{k: bean.id})
            elif clazz == ManyToManyRel:
                getattr(bean, model_field.get_accessor_name()).clear()
                remote_model: VModel = model_field.related_model
                remote_items: QuerySet = remote_model.objects.filter(id__in=value)
                remote_field: ManyToManyField = model_field.remote_field
                remote_field_prop = remote_field.get_attname()
                for _bean in remote_items:
                    bean_set = getattr(_bean, remote_field_prop)
                    bean_set.add(bean)
            elif clazz == OneToOneRel and value is not None:
                remote_model: VModel = model_field.related_model
                remote_field: OneToOneField = model_field.remote_field
                remote_field_prop = remote_field.get_attname()
                _bean = remote_model.objects.get(id=value)
                __bean = remote_model.objects.filter(**{remote_field_prop: bean.id}).first()
                if __bean:
                    setattr(__bean, remote_field_prop, None)
                    __bean.save()
                setattr(_bean, remote_field_prop, bean.id)
                _bean.save()
            elif clazz == OneToOneField and value is not None:
                __bean = model_field.model.objects.filter(**{prop: value}).first()
                if __bean:
                    setattr(__bean, prop, None)
                    __bean.save()
                setattr(bean, prop, value)
            elif clazz == FileField:
                file_name, _bytes = value
                field_file: FieldFile = getattr(bean, prop)
                if field_file:
                    path = field_file.name
                    self.minio.remove(path)
                object_name = self.minio.get_object_name(bean.id, prop, file_name)
                path = self.minio.upload(object_name, _bytes) if _bytes else None
                setattr(bean, prop, path)

    def __get_file_paths__(self, query_set: QuerySet):
        props = self.props('FileField')
        if len(props):
            items = query_set.values(*props)
            paths = []
            for item in items:
                paths += [i for i in item.values() if i]
            return [path for path in paths if path]
        return []

    def __to_dict__(self, query_set: QuerySet, code=None):
        meta_fields = self.fields.values()
        # 简单字段取值
        simple_props = [field.prop for field in meta_fields if field.domain not in __referred_domains__]
        custom_props = __get_custom_props__(self.entity, code)

        results = list(query_set.filter().values(*[*simple_props, *custom_props]))
        __set_simple_values__(meta_fields, results)
        # 关系型字段取值
        mapping = {row['id']: row for row in results}
        referred_fields = [field for field in meta_fields if field.domain in __referred_domains__]
        pks = mapping.keys()
        for meta_field in referred_fields:
            manager: Manager = query_set.model.objects
            qs = manager.filter(id__in=pks)
            self.__linkage__(meta_field, qs, mapping)
        return results

    def __linkage__(self, meta_field: OrmField, query_set: QuerySet, mapping):
        model_field = meta_field.model_field
        prop = model_field.name
        multiple = meta_field.domain in __multiple_domains__

        # 获取级联关系的键索引
        ref_prop = f'{prop}__id'
        edges = query_set.exclude(**{f'{ref_prop}__isnull': True}).values('id', ref_prop)
        if multiple:
            related_primary_keys = set()
            results_mapping = {}
            for edge in edges:
                _id, rid = edge['id'], edge[ref_prop]
                related_primary_keys.add(rid)
                array = results_mapping.get(_id, [])
                array.append(rid)
                results_mapping[_id] = array
        else:
            results_mapping = {row['id']: row[ref_prop] for row in edges if row[ref_prop]}
            related_primary_keys = set(results_mapping.values())

        # 获取级联关系从属方的数据
        related_model = model_field.related_model
        related_fields = related_model._meta.get_fields()
        related_props = self.__get_related_props__(related_fields)
        related_values = list(related_model.objects.filter(id__in=related_primary_keys).values(*related_props))
        __set_simple_values__(related_fields, related_values)
        related_mapping = {item['id']: item for item in related_values}

        # 将从属方的数据绑定在主数据上
        for _id in mapping:
            row = mapping[_id]
            if multiple:
                keys = results_mapping.get(_id, [])
                items = [related_mapping[pid] for pid in keys]
                row[prop] = keys
                row[f'{prop}_set'] = items
            else:
                key = results_mapping.get(_id)
                item = related_mapping.get(key) if key else None
                row[prop] = item
                row[f'{prop}_id'] = key

    def __get_related_props__(self, fields):
        array = []
        for field in fields:
            domain = type(field).__name__
            prop = field.name
            if field.name in __omit_field_props__ or domain in __multiple_domains__:
                continue
            if domain in __referred_domains__:
                field: ForeignKey = field
                model = field.related_model
                meta: Options = model._meta
                if meta.label != self.entity and meta.label != 'valar.Account':
                    _fields = meta.get_fields()
                    for _field in _fields:
                        if type(_field) in [CharField, BooleanField, FloatField, IntegerField, BigAutoField]:
                            array.append(f'{prop}__{_field.name}')
            else:
                array.append(prop)
        return array


""" 以下为静态方法和变量 """

__multiple_domains__ = ['ManyToOneRel', 'ManyToManyField', 'ManyToManyRel']
__referred_domains__ = [*__multiple_domains__, 'OneToOneRel', 'OneToOneField', 'ForeignKey']
__omit_field_props__ = ['create_time', 'modify_time', 'saved', 'sort']
__data_props_formatting__ = {'DateField': '%Y-%m-%d', 'DateTimeField': '%Y-%m-%d %H:%M:%S', 'TimeField': '%H:%M:%S'}


def __get_custom_props__(entity, code='default'):
    field_set = MetaField.objects.filter(view__code=code, view__meta__entity=entity, domain='Custom').values('prop')
    return [item['prop'] for item in field_set if item['prop'] is not None]


def __set_simple_values__(fields, values):
    date_props_mapping = {}
    json_props = []
    for field in fields:
        if isinstance(field, OrmField):
            prop = field.prop
            domain = field.domain
        else:
            prop = field.name
            domain = type(field).__name__
        if domain in __data_props_formatting__.keys():
            date_props_mapping[prop] = __data_props_formatting__[domain]
        elif domain == 'JSONField':
            json_props.append(prop)
    for row in values:
        for prop, formating in date_props_mapping.items():
            if row.get(prop):
                row[prop] = row[prop].strftime(formating)
        for prop in json_props:
            row[prop] = json.loads(row[prop]) if type(row[prop]) is str else row[prop]

    # def fun(field): return type(field).__name__ not in __referred_domains__ and field.name not in __omit_field_props__
    #
    # return [field.name for field in fields if fun(field)]



# Generated by Django 4.2.24 on 2025-10-19 15:26

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='IndicatorSet',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('pid', models.IntegerField(default=0, verbose_name='父节点')),
                ('isLeaf', models.BooleanField(default=False, verbose_name='叶子节点')),
                ('icon', models.CharField(max_length=255, null=True, verbose_name='图标')),
                ('name', models.CharField(max_length=100, null=True)),
            ],
            options={
                'verbose_name': '指标集',
            },
        ),
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('scope', models.CharField(max_length=100, null=True)),
                ('code', models.CharField(max_length=100, null=True)),
                ('name', models.CharField(max_length=100, null=True)),
                ('name_cn', models.CharField(max_length=100, null=True)),
                ('name_en', models.CharField(max_length=100, null=True)),
            ],
            options={
                'verbose_name': '指标集',
            },
        ),
        migrations.CreateModel(
            name='Meta',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('db', models.CharField(max_length=100, null=True, verbose_name='数据库')),
                ('entity', models.CharField(max_length=100, null=True, verbose_name='数据源')),
                ('name', models.CharField(max_length=50, null=True, verbose_name='实体别名')),
                ('tree', models.BooleanField(default=False, verbose_name='是否树形')),
            ],
            options={
                'verbose_name': '数据实体',
                'unique_together': {('db', 'entity')},
            },
        ),
        migrations.CreateModel(
            name='MetaFieldTool',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('pid', models.IntegerField(default=0, verbose_name='父节点')),
                ('isLeaf', models.BooleanField(default=False, verbose_name='叶子节点')),
                ('icon', models.CharField(max_length=255, null=True, verbose_name='图标')),
                ('name', models.CharField(max_length=255, null=True, verbose_name='名称')),
                ('code', models.CharField(max_length=100, null=True, unique=True, verbose_name='代码')),
                ('align', models.CharField(max_length=10, null=True, verbose_name='对齐方式')),
            ],
            options={
                'verbose_name': '元数据字段工具',
            },
        ),
        migrations.CreateModel(
            name='MetaViewDefault',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('name', models.CharField(max_length=50, null=True)),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
            ],
            options={
                'verbose_name': '元数据字段类型',
            },
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('name', models.CharField(max_length=50, null=True)),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('duty', models.TextField(null=True, verbose_name='职责描述')),
            ],
            options={
                'verbose_name': '角色',
            },
        ),
        migrations.CreateModel(
            name='Valar',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('name', models.CharField(max_length=50, null=True)),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('text_field', models.TextField(null=True, verbose_name='Text Field')),
                ('boolean_field', models.BooleanField(null=True, verbose_name='Boolean Field')),
                ('integer_field', models.IntegerField(null=True, verbose_name='Integer Field')),
                ('float_field', models.FloatField(null=True, verbose_name='Float Field')),
                ('date_field', models.DateField(null=True, verbose_name='Date Field')),
                ('datetime_field', models.DateTimeField(null=True, verbose_name='Datetime Field')),
                ('time_field', models.TimeField(null=True, verbose_name='Time Field')),
                ('json_field', models.JSONField(null=True, verbose_name='Json Field')),
                ('file', models.FileField(null=True, upload_to='', verbose_name='File Field')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Voo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('name', models.CharField(max_length=100, null=True, verbose_name='Name')),
                ('valar', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to='valar.valar', verbose_name='Valar')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Vmo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('name', models.CharField(max_length=100, null=True, verbose_name='Name')),
                ('valar', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='valar.valar', verbose_name='Valar')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Vmm',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('pid', models.IntegerField(default=0, verbose_name='父节点')),
                ('isLeaf', models.BooleanField(default=False, verbose_name='叶子节点')),
                ('icon', models.CharField(max_length=255, null=True, verbose_name='图标')),
                ('name', models.CharField(max_length=100, null=True, verbose_name='name')),
                ('valars', models.ManyToManyField(to='valar.valar', verbose_name='valars')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MetaView',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('code', models.CharField(default='default ', max_length=50, verbose_name='类视图')),
                ('name', models.CharField(max_length=50, null=True, verbose_name='视图名称')),
                ('lock', models.BooleanField(default=False, verbose_name='锁定元数据')),
                ('enable', models.BooleanField(default=True, verbose_name='是否启用')),
                ('form_width', models.IntegerField(null=True, verbose_name='表单宽度')),
                ('form_height', models.IntegerField(null=True, verbose_name='表单高度')),
                ('table_width', models.IntegerField(null=True, verbose_name='表格宽度')),
                ('table_height', models.IntegerField(null=True, verbose_name='表格高度')),
                ('allow_search', models.BooleanField(default=True, verbose_name='检索功能')),
                ('allow_order', models.BooleanField(default=True, verbose_name='排序功能')),
                ('allow_insert', models.BooleanField(default=True, verbose_name='新增功能')),
                ('allow_edit', models.BooleanField(default=True, verbose_name='编辑功能')),
                ('allow_edit_on_form', models.BooleanField(default=True, verbose_name='表单编辑')),
                ('allow_edit_on_cell', models.BooleanField(default=True, verbose_name='表内编辑')),
                ('allow_edit_on_sort', models.BooleanField(default=True, verbose_name='移动功能')),
                ('allow_remove', models.BooleanField(default=True, verbose_name='删除功能')),
                ('allow_download', models.BooleanField(default=True, verbose_name='下载功能')),
                ('allow_upload', models.BooleanField(default=True, verbose_name='上传功能')),
                ('meta', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='valar.meta', verbose_name='元数据')),
            ],
            options={
                'verbose_name': '数据视图',
                'unique_together': {('meta', 'code')},
            },
        ),
        migrations.CreateModel(
            name='MetaFieldDomain',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('name', models.CharField(max_length=255, null=True, unique=True, verbose_name='名称')),
                ('default', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='valar.metafieldtool', verbose_name='默认工具')),
                ('tools', models.ManyToManyField(to='valar.metafieldtool', verbose_name='工具集')),
            ],
            options={
                'verbose_name': '元数据字段类型',
            },
        ),
        migrations.CreateModel(
            name='MetaField',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('prop', models.CharField(max_length=100, verbose_name='字段名称')),
                ('label', models.CharField(max_length=100, verbose_name='字段标签')),
                ('name', models.CharField(max_length=100, verbose_name='字段别名')),
                ('domain', models.CharField(max_length=100, verbose_name='字段类型')),
                ('tool', models.CharField(default='default', max_length=100, verbose_name='工具组件')),
                ('refer', models.JSONField(default=dict, verbose_name='索引')),
                ('format', models.JSONField(default=dict, verbose_name='格式')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('not_null', models.BooleanField(default=False, verbose_name='不为空')),
                ('allow_edit', models.BooleanField(default=True, verbose_name='可编辑')),
                ('allow_order', models.BooleanField(default=True, verbose_name='可排序')),
                ('allow_search', models.BooleanField(default=True, verbose_name='可搜索')),
                ('allow_download', models.BooleanField(default=True, verbose_name='可下载')),
                ('allow_upload', models.BooleanField(default=True, verbose_name='可上传')),
                ('allow_update', models.BooleanField(default=True, verbose_name='可更新')),
                ('unit', models.CharField(max_length=55, null=True, verbose_name='单位符')),
                ('column_width', models.FloatField(default=0, verbose_name='表头宽度')),
                ('align', models.CharField(max_length=55, null=True, verbose_name='对齐方式')),
                ('fixed', models.CharField(max_length=100, null=True, verbose_name='固定位置')),
                ('header_color', models.CharField(max_length=55, null=True, verbose_name='表头颜色')),
                ('cell_color', models.CharField(max_length=55, null=True, verbose_name='单元颜色')),
                ('edit_on_table', models.BooleanField(default=True, verbose_name='表格编辑')),
                ('hide_on_table', models.BooleanField(default=False, verbose_name='表内隐藏')),
                ('span', models.IntegerField(default=0, verbose_name='表单占位')),
                ('hide_on_form', models.BooleanField(default=False, verbose_name='表单隐藏')),
                ('hide_on_form_edit', models.BooleanField(default=False, verbose_name='编辑隐藏')),
                ('hide_on_form_insert', models.BooleanField(default=False, verbose_name='新增隐藏')),
                ('hide_on_form_branch', models.BooleanField(default=False, verbose_name='分支隐藏')),
                ('hide_on_form_leaf', models.BooleanField(default=False, verbose_name='叶子隐藏')),
                ('view', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='valar.metaview', verbose_name='数据视图')),
            ],
            options={
                'verbose_name': '视图字段',
            },
        ),
        migrations.CreateModel(
            name='Indicator',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('domain', models.CharField(max_length=100, null=True, verbose_name='domain')),
                ('category', models.CharField(max_length=100, null=True, verbose_name='category')),
                ('name', models.CharField(max_length=100, null=True, verbose_name='名称')),
                ('frequency', models.CharField(max_length=100, null=True, verbose_name='频度')),
                ('scope', models.CharField(max_length=100, null=True, verbose_name='尺度')),
                ('unit', models.CharField(max_length=100, null=True, verbose_name='单位')),
                ('object_id', models.CharField(max_length=100, null=True, verbose_name='ObjectId')),
                ('file', models.FileField(null=True, upload_to='', verbose_name='数据文件')),
                ('set', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='valar.indicatorset', verbose_name='指标集')),
            ],
            options={
                'verbose_name': '指标项',
            },
        ),
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('name', models.CharField(max_length=50, null=True)),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('username', models.CharField(max_length=255, null=True, unique=True, verbose_name='账号')),
                ('password', models.CharField(max_length=255, null=True, verbose_name='密码')),
                ('email', models.CharField(max_length=255, null=True, unique=True, verbose_name='邮箱')),
                ('is_admin', models.BooleanField(default=False, verbose_name='超级管理员')),
                ('is_active', models.BooleanField(default=False, verbose_name='激活状态')),
                ('token', models.CharField(max_length=255, null=True, verbose_name='Token')),
                ('roles', models.ManyToManyField(to='valar.role')),
            ],
            options={
                'verbose_name': '账户信息',
            },
        ),
        migrations.CreateModel(
            name='Menu',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('name', models.CharField(max_length=50, null=True)),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('pid', models.IntegerField(default=0, verbose_name='父节点')),
                ('isLeaf', models.BooleanField(default=False, verbose_name='叶子节点')),
                ('icon', models.CharField(max_length=255, null=True, verbose_name='图标')),
                ('scope', models.CharField(max_length=100, null=True, verbose_name='域')),
                ('path', models.CharField(max_length=255, null=True, verbose_name='地址')),
                ('is_admin', models.BooleanField(default=False, null=True, verbose_name='超管权限')),
                ('is_auth', models.BooleanField(default=False, null=True, verbose_name='需要登录')),
                ('roles', models.ManyToManyField(to='valar.role')),
            ],
            options={
                'verbose_name': '菜单',
                'unique_together': {('scope', 'path')},
            },
        ),
    ]



from django.http import JsonResponse


class ValarResponse(JsonResponse):
    def __init__(self, data=True, message='', code='info', status=200):
        self.valar_message = message
        self.valar_code = code
        self.status_code = status
        super(ValarResponse, self).__init__(data, safe=False)



meta_field_key_defaults = {
    'valar.Meta': {
        'default': ('pick', ['db', 'entity', 'name', 'tree']),
    },
    'valar.MetaView': {
        'list': ('pick', ['code', 'name']),
        'core': ('pick', ['name', 'enable', 'lock']),
        'style': ('pick', ['form_width', 'form_height', 'table_width', 'table_height']),
        'rest': ('pick',
                 ['allow_search', 'allow_order', 'allow_insert', 'allow_remove', 'allow_download', 'allow_upload']),
        'edit': ('pick', ['allow_edit', 'allow_edit_on_form', 'allow_edit_on_cell', 'allow_edit_on_sort']),
    },
    'valar.MetaField': {
        'tool': (
            'pick',
            [
                'name', 'domain', 'tool', 'refer', 'format'
            ]
        ),
        'rest': (
            'pick',
            [
                'name', 'not_null',
                'allow_edit', 'allow_sort', 'allow_search', 'allow_download', 'allow_upload', 'allow_update'
            ]
        ),
        'table': (
            'pick',
            [
                'name', 'unit', 'column_width', 'fixed', 'align', 'edit_on_table', 'hide_on_table',
                'header_color', 'cell_color'
            ]
        ),
        'form': (
            'pick',
            [
                'name', 'span',
                'hide_on_form', 'hide_on_form_insert', 'hide_on_form_edit', 'hide_on_form_branch', 'hide_on_form_leaf'
            ]
        ),
    },
    'valar.Valar': {
        'simple': ('pick', ['id', 'name', 'text_field', 'boolean_field', 'integer_field', 'float_field']),
        'date': ('pick', ['id', 'name', 'date_field', 'datetime_field', 'time_field']),
        'special': ('pick', ['id', 'name', 'text_field', 'json_field', 'file', 'm2m']),
        'ref': ('pick', ['id', 'name', 'm2o', 'm2m', 'o2o_id']),
    },
    'valar.Account': {
        'auth': ('pick', ['user_id', 'username', 'email', 'password', 'is_active', 'is_admin', 'roles']),
    },
    'valar.Role': {
        'auth': ('pick', ['name', 'duty', 'menu', 'account']),
    },
    'valar.Menu': {
        'auth': ('pick', ['icon', 'path', 'name', 'roles', 'is_admin', 'scope']),
    },
    'valar.Indicator': {
        'default': ('pick', ['name', 'frequency', 'scope', 'unit', 'file']),
    },
}



from abc import ABC, abstractmethod

from bson import ObjectId
from django.db.models import QuerySet

from ..dao.engine import ValarEngine
from ..models.core import VModel


class AbstractField(ABC):
    db = None
    entity = None
    prop = None
    label = None
    domain = None
    refer = None
    model_field = None

    @abstractmethod
    def to_dict(self):
        pass


class AbstractDao(ABC):
    engine = ValarEngine()
    db = None
    entity = None
    name = None
    is_tree = False
    fields = {}

    def props(self, domain=None):
        return [prop for prop, field in self.fields.items() if field.domain == domain or domain is None]

    def full_props(self):
        return {
            prop: {
                "prop": prop,
                "domain": field.domain,
                "label": field.label,
            }
            for prop, field in self.fields.items()
        }

    def get_meta_field(self, prop) -> AbstractField:
        return self.fields[prop]

    @abstractmethod
    def save_one(self, item):
        pass

    @abstractmethod
    def insert_one(self, item):
        pass

    @abstractmethod
    def update_one(self, item):
        pass

    @abstractmethod
    def save_many(self, array: list):
        pass

    @abstractmethod
    def values(self, conditions, props) -> list:
        pass

    @abstractmethod
    def delete_one(self, _id):
        pass

    @abstractmethod
    def find_one(self, _id) -> VModel:
        pass

    @abstractmethod
    def find(self, conditions=None, orders=None, size=0, page=1) -> (QuerySet, int):
        pass

    @abstractmethod
    def update(self, template, conditions):
        pass

    @abstractmethod
    def delete(self, conditions=None) -> list:
        pass

    @abstractmethod
    def serialize(self, o, code=None):
        pass

    @abstractmethod
    def tree(self, root, conditions=None):
        pass

    def search(self, includes=None, excludes=None, orders=None):
        conditions = [{"includes": includes or {}, "excludes": excludes or {}}]
        results, _ = self.find(conditions, orders)
        return results

    def object_id(self, _id):
        try:
            return int(_id) if self.db == 'orm' else ObjectId(_id)
        except Exception:
            return None

    # @abstractmethod
    # def values(self, props, conditions, orders=None):
    #     pass
    #
    # @abstractmethod
    # def group(self, props, conditions, orders=None):
    #     pass
    #
    # @abstractmethod
    # def count(self, props, conditions):
    #     pass



from django.apps import apps
from django.conf import settings
from urllib.parse import urlparse

from django.core.mail import EmailMessage
from minio import Minio
from pymongo import MongoClient
from pymongo.synchronous.collection import Collection

from ..classes.singleton_meta import SingletonMeta
from ..classes.valar_minio import ValarMinio
from ..models.core import VModel, VTree


class ValarEngine(metaclass=SingletonMeta):

    def __init__(self):
        orm_engine = {}
        for model in apps.get_models():
            meta = getattr(model, '_meta')
            app = meta.app_label
            if app not in ['sessions']:
                entity = f'{app}.{model.__name__}'
                orm_engine[entity] = model
        self.orm_engine = orm_engine

        # mon
        if hasattr(settings, 'MONGO_URI'):
            self.mongo_client: MongoClient = MongoClient(
                settings.MONGO_URI,
                **{
                    'maxPoolSize': 10,
                    'minPoolSize': 0,
                    'maxIdleTimeMS': 10000,
                    'connectTimeoutMS': 10000,
                    'socketTimeoutMS': 10000,
                    'serverSelectionTimeoutMS': 10000,
                }
            )
            self.mongo_engine = self.mongo_client[settings.BASE_APP]
        else:
            self.mongo_client: MongoClient = None
            self.mongo_engine = None

        # minio
        if hasattr(settings, 'MINIO_URL'):
            parsed = urlparse(settings.MINIO_URL)
            endpoint = f'{parsed.hostname}:{parsed.port}'
            access_key = parsed.username
            secret_key = parsed.password
            self.minio_engine = Minio(
                endpoint=endpoint,
                access_key=access_key,
                secret_key=secret_key,
                secure=False
            )
        else:
            self.minio_engine = None

        self.from_email = settings.EMAIL_HOST_USER

    def send_email(self, title, content, email):
        e = EmailMessage(title, content, self.from_email, [email])
        e.content_subtype = 'html'
        e.send()

    def get_orm_model(self, entity) -> VModel:
        return self.orm_engine[entity]

    def get_mongo_collection(self, entity) -> Collection:
        return self.mongo_engine[entity]

    def get_minio_bucket(self, entity):
        return ValarMinio(self.minio_engine, entity) if self.minio_engine else None

    # def meta_tree(self):
    #     mapping = {}
    #     names = self.mongo_engine.list_collection_names()
    #
    #     for entity, model in self.orm_engine.items():
    #         app, value = entity.split('.')
    #         is_tree = issubclass(model, VTree)
    #         meta = getattr(model, '_meta')
    #         name = meta.verbose_name
    #
    #         node = mapping.get(app, {'label': app, 'value': app, 'db': 'orm', 'children': []})
    #         node['children'].append({"label": name, "value": value, 'isTree': is_tree})
    #         mapping[app] = node
    #     return list(mapping.values())



from ..dao.abstract import AbstractField
from ..dao.frame import meta_field_tool_mapping


class MonField(AbstractField):

    def __init__(self, entity, prop, domain, tool):
        self.db = 'mon'
        self.entity = entity
        self.prop = prop
        self.label = prop
        self.domain = domain
        self.tool = tool

    def to_dict(self):
        _field = {
            "prop": self.prop,
            "label": self.label,
            "name": self.label,
            "domain": self.domain,
            'tool': self.tool,
        }
        return _field



import datetime
import json

import jwt
from django.conf import settings
from django.db.models import OneToOneRel
from django.contrib.auth.hashers import make_password

from ..auth.Authentication import auth_required
from ..classes.valar_response import ValarResponse
from ..dao import Dao
from ..models.auth import Account, AbstractUser, Menu


def sign_in(request):
    body = json.loads(request.body)
    keys = ['username', 'email', 'password', 'signin']
    username, email, password, signin = [body.get(k) for k in keys]
    dao = Dao('valar.Account')
    account = dao.search({"username": username}).first()

    if account is not None:
        if signin:
            if not account.is_auth(password):
                return ValarResponse(False, '密码错误', 'warning')
        else:
            return ValarResponse(False, f"{username}已被占用", 'warning')
    else:
        if signin:
            return ValarResponse(False, f"{username}不存在", 'warning')
        else:
            if username == 'admin' and password != settings.SECRET_KEY:
                return ValarResponse(False, "请输入正确的admin密码", 'warning')
            else:
                account = dao.save_one({
                    "username": username,
                    "email": email,
                    "password": make_password(password),
                    "is_admin": username == 'admin'
                })
    return ValarResponse(jwt.encode({
        "user_id": account.id,
        "exp": datetime.datetime.utcnow() + datetime.timedelta(hours=5)
    }, settings.SECRET_KEY, algorithm="HS256"))


def free_menus(request):
    body = json.loads(request.body)
    scope = body.get("scope", 'admin')
    menus = Menu.objects.filter(
        isLeaf=True, scope=scope, is_admin=False, path__isnull=False, is_auth=False,
        roles__isnull=True).values('path')
    payload = {"permissions": [m['path'] for m in menus]}
    token = jwt.encode(payload, settings.SECRET_KEY, algorithm="HS256")
    return ValarResponse(token)


@auth_required
def user_profile(request):
    body = json.loads(request.body)
    scope = body.get("scope", 'admin')
    account_id = request.user_id
    account = Account.objects.filter(id=account_id).first()
    if account is None:
        return ValarResponse(False, f"账户已不存在", 'warning', status=401)
    user_admin = account.is_admin
    user_roles = list(account.roles.values('id', 'name'))
    user_role_keys = [r['id'] for r in user_roles]
    permissions = []
    for m in Menu.objects.filter(isLeaf=True, scope=scope):
        _id = m.path
        if _id is None:
            continue
        if user_admin:
            permissions.append(_id)
        else:
            if not m.is_admin:
                menu_role_keys = [r['id'] for r in m.roles.all().values('id')]
                if len(menu_role_keys):
                    if bool(set(user_role_keys) & set(menu_role_keys)):
                        permissions.append(_id)
                else:
                    permissions.append(_id)
    payload = {
        "username": account.username,
        "roles": user_roles,
        "is_admin": user_admin,
        "email": account.email,
        "is_active": account.is_active,
        "temporary": account.token is not None,
        "permissions": permissions
    }
    field = (
        next((field for field in account.get_meta().get_fields()
              if type(field) == OneToOneRel
              and issubclass(field.related_model, AbstractUser)), None))
    if field:
        accessor = field.get_accessor_name()
        model = field.related_model
        entity = f'{model._meta.app_label}.{model.__name__}'
        payload.update({'user_entity': entity, 'user_accessor': accessor})
        if hasattr(account, accessor):
            user = getattr(account, accessor)
            name = getattr(user, 'name')
            key = getattr(user, 'id')
            payload.update({'name': name, 'user_key': key})
    token = jwt.encode(payload, settings.SECRET_KEY, algorithm="HS256")
    return ValarResponse(token)



from pymongo.synchronous.collection import Collection
from pymongo.results import InsertOneResult
from pymongo.synchronous.cursor import Cursor

from collections import defaultdict

from .query import Query
from ..dao.abstract import AbstractDao
from ..dao.mon_field import MonField

domain_mapping = {
    'ObjectId': ('BigAutoField', 'none'),
    'int': ('CharField', 'number'),
    'str': ('TextField', 'text'),
    'float': ('CharField', 'number'),
    'bool': ('BooleanField', 'boolean'),
}


class MonDao(AbstractDao):

    def insert_one(self, item):
        pass

    def update_one(self, item):
        pass

    def __init__(self, entity):
        self.db = 'mon'
        self.entity = entity
        self.name = entity
        self.manager: Collection = self.engine.get_mongo_collection(entity)
        self.is_tree = False
        fields = {}
        for prop, domain, tool in self.__analyze_field_types__():
            fields[prop] = MonField(entity, prop, domain, tool)
        self.fields = fields

    def __analyze_field_types__(self):
        array = []
        type_map = defaultdict(set)
        cursor = self.manager.find().limit(1000)
        for doc in cursor:
            for key, value in doc.items():
                type_map[key].add(type(value).__name__)
        for prop, domains in dict(type_map).items():
            domain, tool = domain_mapping[domains.pop()]
            prop = 'id' if prop == '_id' else prop
            array.append((prop, domain, tool))
        return array

    def __detach_item__(self, item):
        _id = item.get('id')
        if _id:
            del item['id']
            return self.object_id(_id), item
        else:
            return None, item

    def save_many(self, array: list):
        self.manager.insert_many(array)

    def values(self, conditions, props):
        return None

    def save_one(self, item):
        oid, item = self.__detach_item__(item)
        if oid:
            self.manager.update_one({'_id': oid}, {'$set': item})
        else:
            bean: InsertOneResult = self.manager.insert_one(item)
            oid = bean.inserted_id
            self.manager.update_one({'_id': oid}, {'$set': {'sort': str(oid)}})
        return self.manager.find_one({'_id': oid})

    def delete_one(self, _id):
        oid = self.object_id(_id)
        flag = oid is not None
        if flag:
            self.manager.delete_one({'_id': oid})
        return flag

    def find_one(self, _id):
        oid = self.object_id(_id)
        return self.manager.find_one({'_id': oid}) if oid else None

    def find(self, conditions=None, orders=None, size=0, page=1):
        finder, orders = Query(conditions, orders).mon()
        skip = (page - 1) * size
        total = self.manager.count_documents(finder)
        cursor = self.manager.find(finder).skip(skip).sort(orders)
        if size:
            cursor = cursor.limit(size)
        return cursor, total

    def update(self, template, conditions):
        flag = template is not None and len(template.keys())
        if flag:
            oid, item = self.__detach_item__(template)
            finder, _ = Query(conditions).mon()
            self.manager.update_many(finder, {'$set': item})
        return flag

    def delete(self, conditions=None) -> list:
        finder, _ = Query(conditions).mon()
        self.manager.delete_many(finder)
        return []

    def serialize(self, o, code=None):
        return [__to_item__(doc) for doc in o] if isinstance(o, Cursor) else __to_item__(o)

    def tree(self, root, conditions=None):
        pass


def __to_item__(o):
    o['id'] = str(o['_id'])
    del o['_id']
    return o
